"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CardanoNodeClient = void 0;
const p_retry_1 = __importDefault(require("p-retry"));
const util_1 = __importStar(require("@cardano-graphql/util"));
const client_1 = require("@cardano-ogmios/client");
const ts_log_1 = require("ts-log");
const util_2 = require("./util");
const MODULE_NAME = 'CardanoNodeClient';
class CardanoNodeClient {
    constructor(logger = ts_log_1.dummyLogger) {
        this.logger = logger;
        this.state = null;
    }
    getTipSlotNo() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'getTipSlotNo');
            }
            const tip = yield this.stateQueryClient.ledgerTip();
            const slotNo = tip === 'origin' ? 0 : tip.slot;
            this.logger.debug({ module: MODULE_NAME, slotNo }, 'getTipSlotNo');
            return slotNo;
        });
    }
    initialize(ogmiosConnectionConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: MODULE_NAME }, 'Initializing. This can take a few minutes...');
            this.serverHealthFetcher = this.serverHealthFetcher || new util_1.DataFetcher('ServerHealth', () => client_1.getServerHealth({ connection: client_1.createConnectionObject(ogmiosConnectionConfig) }), 30000, this.logger);
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.serverHealthFetcher.initialize();
            }), {
                factor: 1.2,
                retries: 3,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Establishing connection to Ogmios server', this.logger)
            });
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const interactionContext = yield util_2.createInteractionContextWithLogger(ogmiosConnectionConfig, this.logger, MODULE_NAME, () => __awaiter(this, void 0, void 0, function* () {
                    yield this.shutdown();
                    yield this.initialize(ogmiosConnectionConfig);
                }));
                this.stateQueryClient = yield client_1.createStateQueryClient(interactionContext);
                this.txSubmissionClient = yield client_1.createTxSubmissionClient(interactionContext);
            }), {
                factor: 1.2,
                retries: 100,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Establishing connection to cardano-node', this.logger)
            });
            this.state = 'initialized';
            this.logger.info({ module: MODULE_NAME }, 'Initialized');
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized')
                return;
            this.logger.info({ module: MODULE_NAME }, 'Shutting down');
            yield this.serverHealthFetcher.shutdown();
            if (this.stateQueryClient.context.socket.readyState === this.stateQueryClient.context.socket.OPEN) {
                yield this.stateQueryClient.shutdown();
            }
            if (this.txSubmissionClient.context.socket.readyState === this.txSubmissionClient.context.socket.OPEN) {
                yield this.txSubmissionClient.shutdown();
            }
            this.state = null;
            this.logger.info({ module: MODULE_NAME }, 'Shutdown');
        });
    }
    submitTransaction(transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'submitTransaction');
            }
            if (this.serverHealthFetcher.value.networkSynchronization < 0.95) {
                throw new util_1.errors.OperationRequiresSyncedNode('submitTransaction');
            }
            const hash = yield this.txSubmissionClient.submitTx(transaction);
            this.logger.info({ module: MODULE_NAME, hash }, 'submitTransaction');
            return hash;
        });
    }
}
exports.CardanoNodeClient = CardanoNodeClient;
//# sourceMappingURL=CardanoNodeClient.js.map