"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChainFollower = void 0;
const client_1 = require("@cardano-ogmios/client");
const p_retry_1 = __importDefault(require("p-retry"));
const util_1 = __importStar(require("@cardano-graphql/util"));
const pg_boss_1 = __importDefault(require("pg-boss"));
const ts_log_1 = require("ts-log");
const util_2 = require("./util");
const MODULE_NAME = 'ChainFollower';
class ChainFollower {
    constructor(hasuraClient, logger = ts_log_1.dummyLogger, queueConfig) {
        this.hasuraClient = hasuraClient;
        this.logger = logger;
        this.queueConfig = queueConfig;
        this.state = null;
    }
    initialize(ogmiosConfig, getMostRecentPoint) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: MODULE_NAME }, 'Initializing');
            this.queue = new pg_boss_1.default(Object.assign({ application_name: 'cardano-graphql' }, this.queueConfig));
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const context = yield util_2.createInteractionContextWithLogger(ogmiosConfig, this.logger, MODULE_NAME, () => __awaiter(this, void 0, void 0, function* () {
                    yield this.shutdown();
                    yield this.initialize(ogmiosConfig, getMostRecentPoint);
                    yield this.start(yield getMostRecentPoint());
                }));
                this.chainSyncClient = yield client_1.createChainSyncClient(context, {
                    rollBackward: ({ point, tip }, requestNext) => __awaiter(this, void 0, void 0, function* () {
                        if (point !== 'origin') {
                            this.logger.info({ module: MODULE_NAME, tip, rollbackPoint: point }, 'Rolling back');
                            const deleteResult = yield this.hasuraClient.deleteAssetsAfterSlot(point.slot);
                            this.logger.info({ module: MODULE_NAME }, `Deleted ${deleteResult} assets`);
                        }
                        else {
                            this.logger.info({ module: MODULE_NAME }, 'Rolling back to genesis');
                            const deleteResult = yield this.hasuraClient.deleteAssetsAfterSlot(0);
                            this.logger.info({ module: MODULE_NAME }, `Deleted ${deleteResult} assets`);
                        }
                        requestNext();
                    }),
                    rollForward: ({ block }, requestNext) => __awaiter(this, void 0, void 0, function* () {
                        let b;
                        if (client_1.isBabbageBlock(block)) {
                            b = block.babbage;
                        }
                        else if (client_1.isAlonzoBlock(block)) {
                            b = block.alonzo;
                        }
                        else if (client_1.isMaryBlock(block)) {
                            b = block.mary;
                        }
                        if (b !== undefined) {
                            for (const tx of b.body) {
                                for (const entry of Object.entries(tx.body.mint.assets)) {
                                    const [policyId, assetName] = entry[0].split('.');
                                    const assetId = `${policyId}${assetName !== undefined ? assetName : ''}`;
                                    if (!(yield this.hasuraClient.hasAsset(assetId))) {
                                        const asset = {
                                            assetId,
                                            assetName,
                                            firstAppearedInSlot: b.header.slot,
                                            fingerprint: util_1.assetFingerprint(policyId, assetName),
                                            policyId
                                        };
                                        yield this.hasuraClient.insertAssets([asset]);
                                        const SIX_HOURS = 21600;
                                        const THREE_MONTHS = 365;
                                        yield this.queue.publish('asset-metadata-fetch-initial', { assetId }, {
                                            retryDelay: SIX_HOURS,
                                            retryLimit: THREE_MONTHS
                                        });
                                    }
                                }
                            }
                        }
                        requestNext();
                    })
                });
            }), {
                factor: 1.2,
                retries: 100,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Establishing connection to cardano-node chain-sync', this.logger)
            });
            this.state = 'initialized';
            this.logger.info({ module: MODULE_NAME }, 'Initialized');
        });
    }
    start(points) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'start');
            }
            this.logger.info({ module: MODULE_NAME }, 'Starting');
            yield this.queue.start();
            yield this.chainSyncClient.startSync(points);
            this.state = 'running';
            this.logger.info({ module: MODULE_NAME }, 'Started');
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'running') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'shutdown');
            }
            this.logger.info({ module: MODULE_NAME }, 'Shutting down');
            yield this.queue.stop();
            if (this.chainSyncClient.context.socket.readyState === this.chainSyncClient.context.socket.OPEN) {
                yield this.chainSyncClient.shutdown();
            }
            this.state = null;
            this.logger.info({ module: MODULE_NAME }, 'Shutdown complete');
        });
    }
}
exports.ChainFollower = ChainFollower;
//# sourceMappingURL=ChainFollower.js.map