"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSchema = exports.scalarResolvers = void 0;
const fs_1 = __importDefault(require("fs"));
const apollo_server_1 = require("apollo-server");
const schema_1 = require("@graphql-tools/schema");
const delegate_1 = require("@graphql-tools/delegate");
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("@cardano-graphql/util"));
const graphql_scalars_1 = require("graphql-scalars");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const queryComplexity_1 = require("./queryComplexity");
const GraphQLBigInt = require('graphql-bigint');
exports.scalarResolvers = {
    AssetFingerprint: util_1.default.scalars.AssetFingerprint,
    BigInt: GraphQLBigInt,
    DateTime: util_1.default.scalars.DateTimeUtcToIso,
    Hash28Hex: util_1.default.scalars.Hash28Hex,
    Hash32Hex: util_1.default.scalars.Hash32Hex,
    Hex: util_1.default.scalars.Hex,
    IPv4: graphql_scalars_1.IPv4Resolver,
    IPv6: graphql_scalars_1.IPv6Resolver,
    JSON: graphql_scalars_1.JSONResolver,
    JSONObject: graphql_scalars_1.JSONObjectResolver,
    Percentage: util_1.default.scalars.Percentage,
    StakeAddress: util_1.default.scalars.StakeAddress,
    StakePoolID: util_1.default.scalars.StakePoolID,
    Timestamp: graphql_scalars_1.TimestampResolver,
    VRFVerificationKey: util_1.default.scalars.VRFVerificationKey
};
function buildSchema(hasuraClient, genesis, cardanoNodeClient, customFieldsComplexity = queryComplexity_1.defaultComplexity) {
    return __awaiter(this, void 0, void 0, function* () {
        const getComplexityExtension = (operation, queryName) => {
            if (operation in customFieldsComplexity) {
                const operationMapping = customFieldsComplexity[operation];
                if (queryName in operationMapping &&
                    operationMapping[queryName].extensions) {
                    return {
                        complexity: operationMapping[queryName].extensions.complexity ||
                            queryComplexity_1.getDefaultQueryComplexity(operationMapping[queryName].extensions.baseCost)
                    };
                }
            }
            return { complexity: queryComplexity_1.getDefaultQueryComplexity() };
        };
        return schema_1.makeExecutableSchema({
            resolvers: Object.assign({}, exports.scalarResolvers, customFieldsComplexity, {
                Mutation: {
                    submitTransaction: {
                        resolve: (_root, args) => __awaiter(this, void 0, void 0, function* () {
                            try {
                                const hash = yield cardanoNodeClient.submitTransaction(args.transaction);
                                return { hash };
                            }
                            catch (error) {
                                if (Array.isArray(error)) {
                                    throw new apollo_server_1.ApolloError('Invalid Transaction', 'BAD_REQUEST', {
                                        reasons: error.map(e => ({
                                            name: e.name,
                                            details: JSON.parse(e.message)
                                        }))
                                    });
                                }
                                else {
                                    throw new apollo_server_1.ApolloError(error);
                                }
                            }
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Mutation', 'submitTransaction')
                    }
                },
                PaymentAddress: {
                    summary: {
                        resolve: (parent, args) => __awaiter(this, void 0, void 0, function* () {
                            try {
                                return yield hasuraClient.getPaymentAddressSummary(parent.address, args.atBlock);
                            }
                            catch (error) {
                                throw new apollo_server_1.ApolloError(error);
                            }
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('PaymentAddress', 'summary')
                    }
                },
                Query: {
                    activeStake: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'activeStake',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'activeStake')
                    },
                    activeStake_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'activeStake_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'activeStake_aggregate')
                    },
                    ada: {
                        resolve: () => __awaiter(this, void 0, void 0, function* () {
                            const adaPots = hasuraClient.adaPotsToCalculateSupplyFetcher.value;
                            if (adaPots === undefined || adaPots === null) {
                                return new apollo_server_1.ApolloError('ada query results are not ready yet. This can occur during startup.');
                            }
                            return {
                                supply: {
                                    circulating: adaPots.circulating,
                                    max: genesis.shelley.maxLovelaceSupply,
                                    total: new bignumber_js_1.default(genesis.shelley.maxLovelaceSupply)
                                        .minus(new bignumber_js_1.default(adaPots.reserves))
                                        .toString()
                                }
                            };
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'ada')
                    },
                    assets: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'assets',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'assets')
                    },
                    assets_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'assets_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'assets_aggregate')
                    },
                    blocks: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'blocks',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'blocks')
                    },
                    blocks_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'blocks_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'blocks_aggregate')
                    },
                    cardano: {
                        resolve: (_root, _args, context, info) => __awaiter(this, void 0, void 0, function* () {
                            var _a;
                            try {
                                const result = yield delegate_1.delegateToSchema({
                                    context,
                                    fieldName: 'cardano',
                                    info,
                                    operation: 'query',
                                    schema: hasuraClient.schema
                                });
                                if (((_a = result[0]) === null || _a === void 0 ? void 0 : _a.currentEpoch) === null) {
                                    return new apollo_server_1.ApolloError('currentEpoch is only available when close to the chain tip. This is expected during the initial chain-sync.');
                                }
                                return result[0];
                            }
                            catch (error) {
                                throw new apollo_server_1.ApolloError(error);
                            }
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'cardano')
                    },
                    cardanoDbMeta: {
                        resolve: () => __awaiter(this, void 0, void 0, function* () {
                            try {
                                const slotNo = yield cardanoNodeClient.getTipSlotNo();
                                return hasuraClient.getMeta(slotNo);
                            }
                            catch (error) {
                                throw new apollo_server_1.ApolloError(error.name === 'ModuleIsNotInitialized'
                                    ? 'cardanoDbMeta query is not ready. Try again shortly'
                                    : error.message);
                            }
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'cardanoDbMeta')
                    },
                    collateralInputs: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'collateralInputs',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'collateralInputs')
                    },
                    collateralInputs_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'collateralInputs_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'collateralInputs_aggregate')
                    },
                    collateralOutputs: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'collateralOutputs',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'collateralOutputs')
                    },
                    collateralOutputs_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'collateralOutputs_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'collateralOutputs_aggregate')
                    },
                    delegations: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'delegations',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'delegations')
                    },
                    delegations_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'delegations_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'delegations_aggregate')
                    },
                    epochs: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'epochs',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'epochs')
                    },
                    epochs_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'epochs_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'epochs_aggregate')
                    },
                    genesis: {
                        resolve: () => __awaiter(this, void 0, void 0, function* () { return genesis; }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'genesis')
                    },
                    paymentAddresses: {
                        resolve: (_root, args) => __awaiter(this, void 0, void 0, function* () {
                            return args.addresses.map((address) => __awaiter(this, void 0, void 0, function* () {
                                return { address };
                            }));
                        }),
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'paymentAddresses')
                    },
                    redeemers: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'redeemers',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'redeemers')
                    },
                    redeemers_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'redeemers_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'redeemers_aggregate')
                    },
                    rewards: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'rewards',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'rewards')
                    },
                    rewards_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'rewards_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'rewards_aggregate')
                    },
                    scripts: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'scripts',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'scripts')
                    },
                    scripts_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'scripts_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'scripts_aggregate')
                    },
                    stakeDeregistrations: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakeDeregistrations',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakeDeregistrations')
                    },
                    stakeDeregistrations_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakeDeregistrations_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakeDeregistrations_aggregate')
                    },
                    stakePools: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakePools',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakePools')
                    },
                    stakePools_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakePools_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakePools_aggregate')
                    },
                    stakeRegistrations: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakeRegistrations',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakeRegistrations')
                    },
                    stakeRegistrations_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'stakeRegistrations_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'stakeRegistrations_aggregate')
                    },
                    transactions: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'transactions',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'transactions')
                    },
                    transactions_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'transactions_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'transactions_aggregate')
                    },
                    tokenMints: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'tokenMints',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'tokenMints')
                    },
                    tokenMints_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'tokenMints_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'tokenMints_aggregate')
                    },
                    utxos: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'utxos',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'utxos')
                    },
                    utxos_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'utxos_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'utxos_aggregate')
                    },
                    withdrawals: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'withdrawals',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'withdrawals')
                    },
                    withdrawals_aggregate: {
                        resolve: (_root, args, context, info) => {
                            return delegate_1.delegateToSchema({
                                args,
                                context,
                                fieldName: 'withdrawals_aggregate',
                                info,
                                operation: 'query',
                                schema: hasuraClient.schema
                            });
                        },
                        selectionSet: null,
                        extensions: getComplexityExtension('Query', 'withdrawals_aggregate')
                    }
                }
            }),
            typeDefs: fs_1.default.readFileSync(path_1.default.resolve(__dirname, '..', 'schema.graphql'), 'utf-8')
        });
    });
}
exports.buildSchema = buildSchema;
//# sourceMappingURL=executableSchema.js.map